pub struct WriterQuadSerializer<W>where
W: Write,{ /* private fields */ }Expand description
Serializes quads or triples to a Write implementation.
Can be built using RdfSerializer::for_writer.
Do not forget to run the finish method to properly write the last bytes of the file.
This writer does unbuffered writes. You might want to use BufWriter to avoid that.
use oxrdfio::{RdfFormat, RdfSerializer};
use oxrdf::{Quad, NamedNode};
let mut serializer = RdfSerializer::from_format(RdfFormat::NQuads).for_writer(Vec::new());
serializer.serialize_quad(&Quad {
subject: NamedNode::new("http://example.com/s")?.into(),
predicate: NamedNode::new("http://example.com/p")?,
object: NamedNode::new("http://example.com/o")?.into(),
graph_name: NamedNode::new("http://example.com/g")?.into(),
})?;
assert_eq!(serializer.finish()?, b"<http://example.com/s> <http://example.com/p> <http://example.com/o> <http://example.com/g> .\n");Implementations