#[non_exhaustive]pub enum UpdateEvaluationError {
Storage(StorageError),
GraphParsing(RdfParseError),
Service(Box<dyn Error + Send + Sync + 'static>),
GraphAlreadyExists(NamedNode),
GraphDoesNotExist(NamedNode),
UnboundService,
InvalidServiceName(Term),
UnsupportedService(NamedNode),
UnsupportedContentType(String),
NotExistingSubstitutedVariable(Variable),
}Expand description
An error from SPARQL UPDATE evaluation
Variants (Non-exhaustive)
This enum is marked as non-exhaustive
Non-exhaustive enums could have additional variants added in future. Therefore, when matching against variants of non-exhaustive enums, an extra wildcard arm must be added to account for any future variants.
Storage(StorageError)
An error from the storage.
GraphParsing(RdfParseError)
An error while parsing an external RDF file.
Service(Box<dyn Error + Send + Sync + 'static>)
Error during SERVICE evaluation
GraphAlreadyExists(NamedNode)
Error when CREATE tries to create an already existing graph
GraphDoesNotExist(NamedNode)
Error when DROP or CLEAR tries to remove a not existing graph
UnboundService
The variable storing the SERVICE name is unbound
InvalidServiceName(Term)
Invalid service name
UnsupportedService(NamedNode)
The given SERVICE is not supported
UnsupportedContentType(String)
The given content media type returned from an HTTP response is not supported (LOAD)
NotExistingSubstitutedVariable(Variable)
If a variable present in the given initial substitution is not present in the SELECT part of the query