pub trait VariableSolutionIndex {
// Required method
fn index(self, solution: &QuerySolution) -> Option<usize>;
}Expand description
A utility trait to get values for a given variable or tuple position.
See QuerySolution::get.
pub trait VariableSolutionIndex {
// Required method
fn index(self, solution: &QuerySolution) -> Option<usize>;
}A utility trait to get values for a given variable or tuple position.
See QuerySolution::get.