PreparedSparqlUpdate

Struct PreparedSparqlUpdate 

pub struct PreparedSparqlUpdate { /* private fields */ }
A prepared SPARQL update.

Usage example:

use oxigraph::sparql::SparqlEvaluator;
use oxigraph::store::Store;

let prepared_update = SparqlEvaluator::new().parse_update(
    "INSERT DATA { <http://example.com> <http://example.com> <http://example.com> }",
)?;
prepared_update.on_store(&Store::new()?).execute()?;

Implementations

Source

impl PreparedSparqlUpdate

pub fn using_datasets(&self) -> impl Iterator<Item = &QueryDatasetSpecification>

pub fn using_datasets_mut( &mut self, ) -> impl Iterator<Item = &mut QueryDatasetSpecification>

pub fn on_store(self, store: &Store) -> BoundPreparedSparqlUpdate<'_, '_>

Bind the prepared update to the Store it should be evaluated on.

Usage example:

use oxigraph::sparql::SparqlEvaluator;
use oxigraph::store::Store;

let prepared_update = SparqlEvaluator::new().parse_update(
    "INSERT DATA { <http://example.com> <http://example.com> <http://example.com> }",
)?;
prepared_update.on_store(&Store::new()?).execute()?;
pub fn on_transaction<'a, 'b: 'a>( self, transaction: &'a mut Transaction<'b>, ) -> BoundPreparedSparqlUpdate<'a, 'b>

Bind the prepared update to the Transaction it should be evaluated on.

Usage example:

use oxigraph::sparql::SparqlEvaluator;
use oxigraph::store::Store;

let prepared_update = SparqlEvaluator::new().parse_update(
    "INSERT DATA { <http://example.com> <http://example.com> <http://example.com> }",
)?;
let store = Store::new()?;
let mut transaction = store.start_transaction()?;
prepared_update.on_transaction(&mut transaction).execute()?;