pub struct PreparedSparqlUpdate { /* private fields */ }Expand description
A prepared SPARQL update.
Usage example:
use oxigraph::sparql::SparqlEvaluator;
use oxigraph::store::Store;
let prepared_update = SparqlEvaluator::new().parse_update(
"INSERT DATA { <http://example.com> <http://example.com> <http://example.com> }",
)?;
prepared_update.on_store(&Store::new()?).execute()?;Implementations
Sourceimpl PreparedSparqlUpdate
impl PreparedSparqlUpdate
Sourcepub fn using_datasets(&self) -> impl Iterator<Item = &QueryDatasetSpecification>
pub fn using_datasets(&self) -> impl Iterator<Item = &QueryDatasetSpecification>
Sourcepub fn using_datasets_mut(
&mut self,
) -> impl Iterator<Item = &mut QueryDatasetSpecification>
pub fn using_datasets_mut( &mut self, ) -> impl Iterator<Item = &mut QueryDatasetSpecification>
Sourcepub fn on_store(self, store: &Store) -> BoundPreparedSparqlUpdate<'_, '_>
pub fn on_store(self, store: &Store) -> BoundPreparedSparqlUpdate<'_, '_>
Bind the prepared update to the Store it should be evaluated on.
Usage example:
use oxigraph::sparql::SparqlEvaluator;
use oxigraph::store::Store;
let prepared_update = SparqlEvaluator::new().parse_update(
"INSERT DATA { <http://example.com> <http://example.com> <http://example.com> }",
)?;
prepared_update.on_store(&Store::new()?).execute()?;Sourcepub fn on_transaction<'a, 'b: 'a>(
self,
transaction: &'a mut Transaction<'b>,
) -> BoundPreparedSparqlUpdate<'a, 'b>
pub fn on_transaction<'a, 'b: 'a>( self, transaction: &'a mut Transaction<'b>, ) -> BoundPreparedSparqlUpdate<'a, 'b>
Bind the prepared update to the Transaction it should be evaluated on.
Usage example:
use oxigraph::sparql::SparqlEvaluator;
use oxigraph::store::Store;
let prepared_update = SparqlEvaluator::new().parse_update(
"INSERT DATA { <http://example.com> <http://example.com> <http://example.com> }",
)?;
let store = Store::new()?;
let mut transaction = store.start_transaction()?;
prepared_update.on_transaction(&mut transaction).execute()?;Trait Implementations
Sourceimpl Clone for PreparedSparqlUpdate
impl Clone for PreparedSparqlUpdate
Sourcefn clone(&self) -> PreparedSparqlUpdate
fn clone(&self) -> PreparedSparqlUpdate
Returns a duplicate of the value. Read more
1.0.0 · Sourcefn clone_from(&mut self, source: &Self)
fn clone_from(&mut self, source: &Self)
Performs copy-assignment from
source. Read moreAuto Trait Implementations
impl Freeze for PreparedSparqlUpdate
impl !RefUnwindSafe for PreparedSparqlUpdate
impl Send for PreparedSparqlUpdate
impl Sync for PreparedSparqlUpdate
impl Unpin for PreparedSparqlUpdate
impl UnsafeUnpin for PreparedSparqlUpdate
impl !UnwindSafe for PreparedSparqlUpdate